import { useState, useEffect } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var getCommentsQuery = "\nquery GetComments($postId: String!, $limit: Int, $offset: Int) {\n  comments(where: {post_id: {_eq: $postId}}, limit: $limit, offset: $offset, order_by: {created_at: desc}) {\n    post_id\n    author\n    content\n    created_at\n  }\n  comments_aggregate(where: {post_id: {_eq: $postId}}) {\n    aggregate {\n      count\n    }\n  }\n}\n";
var addNewCommentMutation = "\nmutation AddNewComment($postId: String!, $author: String!, $content: String!) {\n  insert_comments_one(object: {author: $author, content: $content, post_id: $postId}) {\n    post_id\n    author\n    content\n    created_at\n    hidden\n  }\n}\n";
var errorMessage = 'Oops! Fetching comments was unsuccessful. Try again later.';
/**
 * Fetches comments from Hasura backend specified in `hasuraUrl` on mount and whenever
 * `config.limit` or `config.offset` change.
 *
 * @param hasuraUrl URL of the Hasura instance
 * @param postId Comments will be fetched for the post with id `postId`
 * @param config Configurable offset and limit for the GraphQL query to Hasura
 * @returns comments for giben post, aggregated count of all comments, error,
 *          loading state and a function to refetch data from backend.
 */

var useComments = function useComments(hasuraUrl, postId, config) {
  var _useState = useState([]),
      comments = _useState[0],
      setComments = _useState[1];

  var _useState2 = useState(0),
      count = _useState2[0],
      setCount = _useState2[1];

  var _useState3 = useState(null),
      error = _useState3[0],
      setError = _useState3[1];

  var _useState4 = useState(false),
      loading = _useState4[0],
      setLoading = _useState4[1];

  var fetchComments = function fetchComments() {
    setLoading(true);
    fetch(hasuraUrl, {
      method: 'POST',
      headers: {
        'x-hasura-role': 'anonymous'
      },
      body: JSON.stringify({
        query: getCommentsQuery,
        variables: _extends({
          postId: postId
        }, (config == null ? void 0 : config.limit) && {
          limit: config.limit
        }, (config == null ? void 0 : config.offset) && {
          offset: config.offset
        })
      })
    }).then(function (res) {
      return res.json();
    }).then(function (res) {
      if (res.errors && res.errors.length) {
        setError({
          error: errorMessage,
          details: res.errors[0].message
        });
        setLoading(false);
        return;
      }

      setComments(res.data.comments);
      setCount(res.data.comments_aggregate.aggregate.count);
      setLoading(false);
    })["catch"](function (err) {
      setError({
        error: errorMessage,
        details: err
      });
      setLoading(false);
    });
  };

  useEffect(fetchComments, [config == null ? void 0 : config.limit, config == null ? void 0 : config.offset]);

  var addComment = function addComment(_ref) {
    var content = _ref.content,
        author = _ref.author;
    var createdAt = new Date().toDateString();
    var newComment = {
      author: author,
      content: content,
      post_id: postId,
      created_at: createdAt,
      status: 'sending'
    };
    setComments(function (prev) {
      return [newComment].concat(prev);
    });
    setCount(function (prev) {
      return prev++;
    });
    fetch(hasuraUrl, {
      method: 'POST',
      headers: {
        'x-hasura-role': 'user'
      },
      body: JSON.stringify({
        query: addNewCommentMutation,
        variables: {
          postId: postId,
          content: content,
          author: author
        }
      })
    }).then(function (res) {
      return res.json();
    }).then(function (res) {
      if (res.errors && res.errors.length) {
        setError({
          error: errorMessage,
          details: res.errors[0].message
        });
        return;
      }

      var remoteComment = res.data.insert_comments_one;
      setComments(function (prev) {
        return prev.map(function (x) {
          return x === newComment ? _extends({}, remoteComment, {
            status: remoteComment.hidden ? 'delivered-awaiting-approval' : 'added'
          }) : x;
        });
      });
    })["catch"](function (err) {
      setError({
        error: errorMessage,
        details: err
      });
      setComments(function (prev) {
        return prev.map(function (x) {
          return x === newComment ? _extends({}, newComment, {
            status: 'failed'
          }) : x;
        });
      });
    });
  };

  return {
    comments: comments,
    addComment: addComment,
    refetch: fetchComments,
    count: count,
    loading: loading,
    error: error
  };
};

export { useComments };
//# sourceMappingURL=index.esm.js.map
